#include <stdio.h>
#include <queue>
#include <vector>
using namespace std;
typedef long long llong;
const int MAXN = 500011;

int n;
int a[MAXN];

struct Node {
    int minIdx, maxIdx;
    pair<int, int> diffIdx;
    llong minVal = 0, maxVal = 0;
    llong bestDiff = -1;

    bool operator<(const Node& other) const {
        return bestDiff < other.bestDiff;
    }
};

Node mergeNodes(const Node& A, const Node& B) {
    if(A.bestDiff == -1) {
        return B;
    }
    if(B.bestDiff == -1) {
        return A;
    }

    Node C;

    if(A.minVal < B.minVal) {
        C.minVal = A.minVal;
        C.minIdx = A.minIdx;
    } else {
        C.minVal = B.minVal;
        C.minIdx = B.minIdx;
    }

    if(A.maxVal > B.maxVal) {
        C.maxVal = A.maxVal;
        C.maxIdx = A.maxIdx;
    } else {
        C.maxVal = B.maxVal;
        C.maxIdx = B.maxIdx;
    }

    if(A.bestDiff > B.bestDiff) {
        C.bestDiff = A.bestDiff;
        C.diffIdx = A.diffIdx;
    } else {
        C.bestDiff = B.bestDiff;
        C.diffIdx = B.diffIdx;
    }

    if(B.maxVal - A.minVal > C.bestDiff) {
        C.bestDiff = B.maxVal - A.minVal;
        C.diffIdx = {A.minIdx, B.maxIdx};
    }

    return C;
}

Node ITpos[4 * MAXN];
Node ITneg[4 * MAXN];
int LEAFOFFSET = 1;

vector<pair<int, int>> ranges;
vector<bool> isPos;

priority_queue<pair<Node, int>> pq;

Node recBest(int ver, int L, int R, int l, int r, const Node IT[]) {
    if(L > r || R < l) {
        return Node();
    } else if(L >= l && R <= r) {
        return IT[ver];
    } else {
        return mergeNodes(
            recBest(2 * ver, L, (L + R) / 2, l, r, IT),
            recBest(2 * ver + 1, (L + R) / 2 + 1, R, l, r, IT)
        );
    }
}

Node getBest(int L, int R, const Node IT[]) {
    return recBest(1, 1, LEAFOFFSET + 1, L, R, IT);
}

Node getResult(int rangeId) {
    if(isPos[rangeId]) {
        return getBest(ranges[rangeId].first, ranges[rangeId].second, ITpos);
    } else {
        return getBest(ranges[rangeId].first, ranges[rangeId].second, ITneg);
    }
}

int main() {
    scanf("%d", &n);

    LEAFOFFSET = 1;
    while(LEAFOFFSET < n) {
        LEAFOFFSET *= 2;
    }
    LEAFOFFSET--;

    for(int i = 1; i <= n; i++) {
        scanf("%d", &a[i]);

        ITpos[i + LEAFOFFSET].minVal = a[i];
        ITpos[i + LEAFOFFSET].minIdx = i;
        ITpos[i + LEAFOFFSET].maxVal = a[i];
        ITpos[i + LEAFOFFSET].maxIdx = i;
        ITpos[i + LEAFOFFSET].bestDiff = 0;
        ITpos[i + LEAFOFFSET].diffIdx = {i, i};

        ITneg[i + LEAFOFFSET].minVal = -a[i];
        ITneg[i + LEAFOFFSET].maxVal = -a[i];
        ITneg[i + LEAFOFFSET].minIdx = i;
        ITneg[i + LEAFOFFSET].maxIdx = i;
        ITneg[i + LEAFOFFSET].diffIdx = {i, i};
        ITneg[i + LEAFOFFSET].bestDiff = 0;
    }

    for(int i = LEAFOFFSET; i >= 1; i--) {
        ITpos[i] = mergeNodes(ITpos[2 * i], ITpos[2 * i + 1]);
        ITneg[i] = mergeNodes(ITneg[2 * i], ITneg[2 * i + 1]);
    }

    ranges.push_back({1, n});
    isPos.push_back(true);
    pq.push({getResult(0), 0});

    llong answer = 0;
    for(int i = 1; i <= n; i++) {
        auto [node, id] = pq.top();
        pq.pop();

        answer += node.bestDiff;
        printf("%lld", answer);
        if(i != n) {
            printf(" ");
        }

        auto [rangeL, rangeR] = ranges[id];
        auto [choiceL, choiceR] = node.diffIdx;

        if(choiceL != rangeL) {
            ranges.push_back({rangeL, choiceL - 1});
            isPos.push_back(isPos[id]);
            pq.push({getResult(ranges.size() - 1), ranges.size() - 1});
        }

        ranges.push_back({choiceL, choiceR});
        isPos.push_back(!isPos[id]);
        pq.push({getResult(ranges.size() - 1), ranges.size() - 1});

        if(choiceR != rangeR) {
            ranges.push_back({choiceR + 1, rangeR});
            isPos.push_back(isPos[id]);
            pq.push({getResult(ranges.size() - 1), ranges.size() - 1});
        }
    }
    printf("\n");

    return 0;
}
