#include <bits/stdc++.h>
// #include <coding_library/data_structures/segment_tree_lazy.hpp>
// #include <coding_library/data_structures/monoids_lazy.hpp>

using namespace std;

template<typename T1, typename T2>
ostream& operator<<(ostream& out, const pair<T1, T2>& x) {
    return out << x.first << ' ' << x.second;
}

template<typename T1, typename T2>
istream& operator>>(istream& in, pair<T1, T2>& x) {
    return in >> x.first >> x.second;
}

template<typename T>
istream& operator>>(istream& in, vector<T>& a) {
    for(auto& x: a) {
        in >> x;
    }
    return in;
};

template<typename T>
ostream& operator<<(ostream& out, const vector<T>& a) {
    for(auto x: a) {
        out << x << ' ';
    }
    return out;
};

template<
    class T, T (*merge)(T, T), T (*e)(), class G, T (*lazy_apply)(G, T),
    G (*lazy_merge)(G, G), G (*lazy_init)(T)>
class SegmentTreeLazy {
  private:
    int n, size, log_size;
    vector<T> tr;
    vector<G> lazy;

    void pull(int x) { tr[x] = merge(tr[2 * x], tr[2 * x + 1]); }

    void push_one(int x, G f) {
        tr[x] = lazy_apply(f, tr[x]);
        if(x < size) {
            lazy[x] = lazy_merge(f, lazy[x]);
        }
    }

    void push(int x) {
        push_one(2 * x, lazy[x]);
        push_one(2 * x + 1, lazy[x]);
        lazy[x] = lazy_init(tr[x]);
    }

    void push_all_down(int x) {
        for(int i = log_size; i >= 1; i--) {
            push(x >> i);
        }
    }

    void push_all_open_range(int l, int r) {
        for(int i = log_size; i >= 1; i--) {
            if(((l >> i) << i) != l) {
                push(l >> i);
            }
            if(((r >> i) << i) != r) {
                push((r - 1) >> i);
            }
        }
    }

  public:
    SegmentTreeLazy() { init(vector<T>()); }
    SegmentTreeLazy(int _n) { init(vector<T>(_n, e())); }
    SegmentTreeLazy(const vector<T>& _a) { init(_a); }

    void init(const vector<T>& _a) {
        n = _a.size();
        size = 1;
        log_size = 0;
        while(size < n) {
            size <<= 1;
            log_size++;
        }

        tr.assign(2 * size, e());
        lazy.resize(size);
        for(int i = 0; i < n; i++) {
            tr[size + i] = _a[i];
        }
        for(int i = size - 1; i > 0; i--) {
            pull(i);
            lazy[i] = lazy_init(tr[i]);
        }
    }

    void update(int pos, T val) {
        pos += size;
        push_all_down(pos);
        tr[pos] = val;
        for(pos >>= 1; pos > 0; pos >>= 1) {
            pull(pos);
        }
    }

    void apply_lazy(int l, int r, G f) {
        if(l > r) {
            return;
        }

        l += size, r += size + 1;
        push_all_open_range(l, r);
        for(int l0 = l, r0 = r; l0 < r0; l0 >>= 1, r0 >>= 1) {
            if(l0 & 1) {
                push_one(l0++, f);
            }
            if(r0 & 1) {
                push_one(--r0, f);
            }
        }

        for(int i = 1; i <= log_size; i++) {
            if(((l >> i) << i) != l) {
                pull(l >> i);
            }
            if(((r >> i) << i) != r) {
                pull((r - 1) >> i);
            }
        }
    }

    T get_pos(int pos) {
        pos += size;
        push_all_down(pos);
        return tr[pos];
    }

    T query(int l, int r) {
        T ansl = e(), ansr = e();
        l += size, r += size + 1;
        push_all_open_range(l, r);

        for(; l < r; l >>= 1, r >>= 1) {
            if(l & 1) {
                ansl = merge(ansl, tr[l++]);
            }
            if(r & 1) {
                ansr = merge(tr[--r], ansr);
            }
        }
        return merge(ansl, ansr);
    }

    T query_all() { return tr[1]; }

    template<bool (*f)(T)>
    int max_right(int l) const {
        return max_right(l, [](T x) { return f(x); });
    }
    template<class F>
    int max_right(int l, F f) const {
        if(l == n) {
            return n;
        }

        l += size;
        push_all_down(l);
        T sm = e();
        do {
            while(l % 2 == 0) {
                l >>= 1;
            }
            if(!f(merge(sm, tr[l]))) {
                while(l < size) {
                    push(l);
                    l = (2 * l);
                    if(f(merge(sm, tr[l]))) {
                        sm = merge(sm, tr[l]);
                        l++;
                    }
                }
                return l - size;
            }
            sm = merge(sm, tr[l]);
            l++;
        } while((l & -l) != l);
        return n;
    }

    template<bool (*f)(T)>
    int min_left(int r) const {
        return min_left(r, [](T x) { return f(x); });
    }
    template<class F>
    int min_left(int r, F f) const {
        if(r == -1) {
            return 0;
        }

        r += size + 1;
        push_all_down(r - 1);
        T sm = e();
        do {
            r--;
            while(r > 1 && (r % 2)) {
                r >>= 1;
            }
            if(!f(merge(tr[r], sm))) {
                while(r < size) {
                    push(r);
                    r = (2 * r + 1);
                    if(f(merge(tr[r], sm))) {
                        sm = merge(tr[r], sm);
                        r--;
                    }
                }
                return r + 1 - size;
            }
            sm = merge(tr[r], sm);
        } while((r & -r) != r);
        return 0;
    }
};

template<class T>
class MonoidMaxSubarray {
  public:
    struct Sum {
        int l, r;  // Segment boundaries
        T sum;     // Total sum of the segment

        T max_subarray_sum;  // Maximum subarray sum
        int l_max, r_max;    // Boundaries of max subarray

        T min_subarray_sum;  // Minimum subarray sum
        int l_min, r_min;    // Boundaries of min subarray

        T max_prefix_sum;  // Maximum prefix sum
        T max_prefix_len;  // Length of max prefix

        T min_prefix_sum;  // Minimum prefix sum
        T min_prefix_len;  // Length of min prefix

        T max_suffix_sum;  // Maximum suffix sum
        T max_suffix_len;  // Length of max suffix

        T min_suffix_sum;  // Minimum suffix sum
        T min_suffix_len;  // Length of min suffix

        bool flipped;  // Whether this segment is flipped
    };

    struct NegateMultiplyLazy {
        bool negate;  // Whether to negate the values
    };

    static Sum e() {
        return {
            0,   // l
            -1,  // r
            0,   // sum

            0,   // max_subarray_sum
            0,   // l_max
            -1,  // r_max

            0,   // min_subarray_sum
            0,   // l_min
            -1,  // r_min

            0,  // max_prefix_sum
            0,  // max_prefix_len

            0,  // min_prefix_sum
            0,  // min_prefix_len

            0,  // max_suffix_sum
            0,  // max_suffix_len

            0,  // min_suffix_sum
            0,  // min_suffix_len

            false  // flipped
        };
    }

    static Sum merge(Sum a, Sum b) {
        if(a.l == 0 && a.r == -1) {
            return b;
        }
        if(b.l == 0 && b.r == -1) {
            return a;
        }

        Sum res;
        res.l = a.l;
        res.r = b.r;
        res.sum = a.sum + b.sum;
        res.flipped = false;

        // Calculate max subarray sum (either in a, in b, or crossing a and b)
        T crossing_max = a.max_suffix_sum + b.max_prefix_sum;

        if(a.max_subarray_sum >= b.max_subarray_sum &&
           a.max_subarray_sum >= crossing_max) {
            res.max_subarray_sum = a.max_subarray_sum;
            res.l_max = a.l_max;
            res.r_max = a.r_max;
        } else if(b.max_subarray_sum >= a.max_subarray_sum && b.max_subarray_sum >= crossing_max) {
            res.max_subarray_sum = b.max_subarray_sum;
            res.l_max = b.l_max;
            res.r_max = b.r_max;
        } else {
            res.max_subarray_sum = crossing_max;
            res.l_max = a.r - a.max_suffix_len + 1;
            res.r_max = b.l + b.max_prefix_len - 1;
        }

        // Calculate min subarray sum (either in a, in b, or crossing a and b)
        T crossing_min = a.min_suffix_sum + b.min_prefix_sum;

        if(a.min_subarray_sum <= b.min_subarray_sum &&
           a.min_subarray_sum <= crossing_min) {
            res.min_subarray_sum = a.min_subarray_sum;
            res.l_min = a.l_min;
            res.r_min = a.r_min;
        } else if(b.min_subarray_sum <= a.min_subarray_sum && b.min_subarray_sum <= crossing_min) {
            res.min_subarray_sum = b.min_subarray_sum;
            res.l_min = b.l_min;
            res.r_min = b.r_min;
        } else {
            res.min_subarray_sum = crossing_min;
            res.l_min = a.r - a.min_suffix_len + 1;
            res.r_min = b.l + b.min_prefix_len - 1;
        }

        // Calculate max prefix sum
        if(a.max_prefix_sum >= a.sum + b.max_prefix_sum) {
            res.max_prefix_sum = a.max_prefix_sum;
            res.max_prefix_len = a.max_prefix_len;
        } else {
            res.max_prefix_sum = a.sum + b.max_prefix_sum;
            res.max_prefix_len = a.r - a.l + 1 + b.max_prefix_len;
        }

        // Calculate min prefix sum
        if(a.min_prefix_sum <= a.sum + b.min_prefix_sum) {
            res.min_prefix_sum = a.min_prefix_sum;
            res.min_prefix_len = a.min_prefix_len;
        } else {
            res.min_prefix_sum = a.sum + b.min_prefix_sum;
            res.min_prefix_len = a.r - a.l + 1 + b.min_prefix_len;
        }

        // Calculate max suffix sum
        if(b.max_suffix_sum >= b.sum + a.max_suffix_sum) {
            res.max_suffix_sum = b.max_suffix_sum;
            res.max_suffix_len = b.max_suffix_len;
        } else {
            res.max_suffix_sum = b.sum + a.max_suffix_sum;
            res.max_suffix_len = b.r - b.l + 1 + a.max_suffix_len;
        }

        // Calculate min suffix sum
        if(b.min_suffix_sum <= b.sum + a.min_suffix_sum) {
            res.min_suffix_sum = b.min_suffix_sum;
            res.min_suffix_len = b.min_suffix_len;
        } else {
            res.min_suffix_sum = b.sum + a.min_suffix_sum;
            res.min_suffix_len = b.r - b.l + 1 + a.min_suffix_len;
        }

        return res;
    }

    static Sum lazy_apply(NegateMultiplyLazy f, Sum x) {
        if(!f.negate) {
            return x;
        }

        // Negate all values and swap min/max
        Sum res = x;
        res.sum = -x.sum;

        // Swap max and min subarray
        std::swap(res.max_subarray_sum, res.min_subarray_sum);
        res.max_subarray_sum = -res.max_subarray_sum;
        res.min_subarray_sum = -res.min_subarray_sum;

        std::swap(res.l_max, res.l_min);
        std::swap(res.r_max, res.r_min);

        // Swap max and min prefix
        std::swap(res.max_prefix_sum, res.min_prefix_sum);
        res.max_prefix_sum = -res.max_prefix_sum;
        res.min_prefix_sum = -res.min_prefix_sum;

        std::swap(res.max_prefix_len, res.min_prefix_len);

        // Swap max and min suffix
        std::swap(res.max_suffix_sum, res.min_suffix_sum);
        res.max_suffix_sum = -res.max_suffix_sum;
        res.min_suffix_sum = -res.min_suffix_sum;

        std::swap(res.max_suffix_len, res.min_suffix_len);

        // Toggle flipped flag
        res.flipped = !res.flipped;

        return res;
    }

    static NegateMultiplyLazy lazy_merge(
        NegateMultiplyLazy a, NegateMultiplyLazy b
    ) {
        // XOR the negate flags
        return {(bool)(a.negate ^ b.negate)};
    }

    static NegateMultiplyLazy lazy_init(Sum node) {
        return {false};  // Initially, no negation
    }

    // Helper function to create a leaf node
    static Sum make_leaf(T value, T position) {
        Sum leaf;
        leaf.l = position;
        leaf.r = position;
        leaf.sum = value;
        leaf.flipped = false;

        // For a single element, all subarrays are the same
        if(value > 0) {
            leaf.max_subarray_sum = value;
            leaf.min_subarray_sum = 0;
            leaf.l_max = position;
            leaf.r_max = position;
            leaf.l_min = position;
            leaf.r_min = position - 1;
        } else {
            leaf.max_subarray_sum = 0;
            leaf.min_subarray_sum = value;
            leaf.l_max = position;
            leaf.r_max = position - 1;
            leaf.l_min = position;
            leaf.r_min = position;
        }


        // Prefix calculations
        leaf.max_prefix_sum = std::max<T>(0, value);
        leaf.min_prefix_sum = std::min<T>(0, value);
        leaf.max_prefix_len = (value > 0) ? 1 : 0;
        leaf.min_prefix_len = (value < 0) ? 1 : 0;

        // Suffix calculations
        leaf.max_suffix_sum = std::max<T>(0, value);
        leaf.min_suffix_sum = std::min<T>(0, value);
        leaf.max_suffix_len = (value > 0) ? 1 : 0;
        leaf.min_suffix_len = (value < 0) ? 1 : 0;

        return leaf;
    }
};

using MonoidType = MonoidMaxSubarray<int64_t>;
using SegTree = SegmentTreeLazy<
    MonoidType::Sum, MonoidType::merge, MonoidType::e,
    MonoidType::NegateMultiplyLazy, MonoidType::lazy_apply,
    MonoidType::lazy_merge, MonoidType::lazy_init>;

int n;
vector<int> price;

void read() {
    cin >> n;
    price.resize(n);
    cin >> price;
}

// void print_node(MonoidType::Sum mt) {
//     cout << "printing node..." << endl;
//     cout << mt.l << " " << mt.r << endl;
//     cout << mt.sum << endl;
//     cout << mt.max_prefix_len << endl;
//     cout << mt.min_prefix_len << endl;
//     cout << "ans:" << endl;
//     cout << mt.max_subarray_sum << endl; 
//     cout << mt.l_max << " " << mt.r_max << endl; 
//     cout << mt.min_subarray_sum << endl; 
//     cout << mt.l_min << " " << mt.r_min << endl; 
// }

void solve() {
    vector<int> a(n - 1);
    for(int i = 1; i < n; i++) {
        a[i - 1] = price[i] - price[i - 1];
    }
 
    vector<MonoidType::Sum> initial(n - 1);
    for(int i = 0; i < n - 1; i++) {
        initial[i] = MonoidType::make_leaf(a[i], i);
    }

    SegTree t(initial);

    int64_t ans = 0, global_opt = 0;
    for(int k = 1; k <= n; k++) {
        auto best = t.query_all();
        ans += best.max_subarray_sum;

        // cout << endl;
        // cout << endl;
        // cout << best.l_max << " " << best.r_max << endl;
        // cout << "-> " << best.max_subarray_sum << endl;
        // print_node(t.query(0, 0));
    
        t.apply_lazy(best.l_max, best.r_max, {true});
        global_opt = max(global_opt, ans);
        cout << global_opt << " ";
    }

    cout << endl;
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int T = 1;
    // cin >> T;
    for(int test = 1; test <= T; test++) {
        read();
        // cout << "Case #" << test << ": ";
        solve();
    }

    return 0;
}
